Merge "ref: Remove support for API versions <9 and endpoint versions < 5."
diff --git a/cmd/principal/doc.go b/cmd/principal/doc.go
index 63218fe..57d8f83 100644
--- a/cmd/principal/doc.go
+++ b/cmd/principal/doc.go
@@ -443,11 +443,12 @@
 this tool is running in.
 
 The key is printed as a base64 encoded bytes of the DER-format representation of
-the key.
+the key (suitable to be provided as an argument to the 'addtoroots' command for
+example).
 
-If the --pretty flag is provided, then the key is printed in the XX:XX:...:XX
-format typically used in the output of other commands. This representation,
-while prettier, is lossy.
+With --pretty, a 16-byte fingerprint of the key instead. This format is easier
+for humans to read and is used in output of other commands in this program, but
+is not suitable as an argument to the 'addtoroots' command.
 
 Usage:
    principal get publickey [flags]
diff --git a/cmd/principal/main.go b/cmd/principal/main.go
index 3602c34..05df6ee 100644
--- a/cmd/principal/main.go
+++ b/cmd/principal/main.go
@@ -301,11 +301,12 @@
 that this tool is running in.
 
 The key is printed as a base64 encoded bytes of the DER-format representation
-of the key.
+of the key (suitable to be provided as an argument to the 'addtoroots' command
+for example).
 
-If the --pretty flag is provided, then the key is printed in the XX:XX:...:XX
-format typically used in the output of other commands. This representation,
-while prettier, is lossy.
+With --pretty, a 16-byte fingerprint of the key instead. This format is easier
+for humans to read and is used in output of other commands in this program, but
+is not suitable as an argument to the 'addtoroots' command.
 `,
 		Run: func(cmd *cmdline.Command, args []string) error {
 			ctx, shutdown := v23.Init()
diff --git a/lib/vdl/build/build.go b/lib/vdl/build/build.go
index 397ac63..4bc5ce2 100644
--- a/lib/vdl/build/build.go
+++ b/lib/vdl/build/build.go
@@ -373,7 +373,7 @@
 	// This special env is used when building "vdl.config" files, which have the
 	// implicit "vdltool" import.
 	if !ds.ResolvePath("vdltool", UnknownPathIsError) {
-		errs.Errorf(`Can't resolve "vdltool" package`)
+		errs.Errorf(`can't resolve "vdltool" package`)
 	}
 	for _, pkg := range ds.Sort() {
 		BuildPackage(pkg, ds.vdlenv)
@@ -409,7 +409,7 @@
 	case isDirPath:
 		return ds.resolveDirPath(path, mode) != nil
 	default:
-		return ds.resolveImportPath(path, mode) != nil
+		return ds.resolveImportPath(path, mode, "") != nil
 	}
 }
 
@@ -508,7 +508,7 @@
 	rePat = `^` + rePat + `$`
 	matcher, err := regexp.Compile(rePat)
 	if err != nil {
-		return nil, fmt.Errorf("Can't compile package path regexp %s: %v", rePat, err)
+		return nil, fmt.Errorf("can't compile package path regexp %s: %v", rePat, err)
 	}
 	return matcher, nil
 }
@@ -558,18 +558,18 @@
 
 // resolveImportPath resolves pkgPath into a Package.  Returns the package, or
 // nil if it can't be resolved.
-func (ds *depSorter) resolveImportPath(pkgPath string, mode UnknownPathMode) *Package {
+func (ds *depSorter) resolveImportPath(pkgPath string, mode UnknownPathMode, prefix string) *Package {
 	pkgPath = path.Clean(pkgPath)
 	if pkg := ds.pathMap[pkgPath]; pkg != nil {
 		return pkg
 	}
 	if !validPackagePath(pkgPath) {
-		mode.logOrErrorf(ds.errs, "Import path %q is invalid", pkgPath)
+		mode.logOrErrorf(ds.errs, "%s: import path %q is invalid", prefix, pkgPath)
 		return nil
 	}
 	// Special-case to disallow packages under the vdlroot dir.
 	if strings.HasPrefix(pkgPath, vdlrootImportPrefix) {
-		mode.logOrErrorf(ds.errs, "Import path %q is invalid (packages under vdlroot must be specified without the vdlroot prefix)", pkgPath)
+		mode.logOrErrorf(ds.errs, "%s: import path %q is invalid (packages under vdlroot must be specified without the vdlroot prefix)", prefix, pkgPath)
 		return nil
 	}
 	// Look through srcDirs in-order until we find a valid package dir.
@@ -584,7 +584,7 @@
 	}
 	// We can't find a valid dir corresponding to this import path.
 	detail := "   " + strings.Join(dirs, "\n   ")
-	mode.logOrErrorf(ds.errs, "Can't resolve import path %q in any of:\n%s", pkgPath, detail)
+	mode.logOrErrorf(ds.errs, "%s: can't resolve %q in any of:\n%s", prefix, pkgPath, detail)
 	return nil
 }
 
@@ -601,7 +601,7 @@
 	pfiles := ParsePackage(pkg, parse.Opts{ImportsOnly: true}, ds.errs)
 	pkg.Name = parse.InferPackageName(pfiles, ds.errs)
 	for _, pf := range pfiles {
-		ds.addImportDeps(pkg, pf.Imports)
+		ds.addImportDeps(pkg, pf.Imports, filepath.Join(path, pf.BaseName))
 	}
 	return pkg
 }
@@ -609,9 +609,9 @@
 // addImportDeps adds transitive dependencies represented by imports to the
 // sorter.  If the pkg is non-nil, an edge is added between the pkg and its
 // dependencies; otherwise each dependency is added as an independent node.
-func (ds *depSorter) addImportDeps(pkg *Package, imports []*parse.Import) {
+func (ds *depSorter) addImportDeps(pkg *Package, imports []*parse.Import, file string) {
 	for _, imp := range imports {
-		if dep := ds.resolveImportPath(imp.Path, UnknownPathIsError); dep != nil {
+		if dep := ds.resolveImportPath(imp.Path, UnknownPathIsError, file); dep != nil {
 			if pkg != nil {
 				ds.sorter.AddEdge(pkg, dep)
 			} else {
@@ -625,7 +625,7 @@
 // and adds all transitive dependencies to the sorter.
 func (ds *depSorter) AddConfigDeps(fileName string, src io.Reader) {
 	if pconfig := parse.ParseConfig(fileName, src, parse.Opts{ImportsOnly: true}, ds.errs); pconfig != nil {
-		ds.addImportDeps(nil, pconfig.Imports)
+		ds.addImportDeps(nil, pconfig.Imports, fileName)
 	}
 }
 
@@ -657,7 +657,7 @@
 	sorted, cycles := ds.sorter.Sort()
 	if len(cycles) > 0 {
 		cycleStr := toposort.DumpCycles(cycles, printPackagePath)
-		ds.errorf("Cyclic package dependency: %v", cycleStr)
+		ds.errorf("cyclic package dependency: %v", cycleStr)
 		return nil
 	}
 	if len(sorted) == 0 {
@@ -726,7 +726,7 @@
 	ds := newDepSorter(opts, errs)
 	for _, path := range paths {
 		if !ds.ResolvePath(path, mode) {
-			mode.logOrErrorf(errs, "Can't resolve %q to any packages", path)
+			mode.logOrErrorf(errs, "can't resolve %q to any packages", path)
 		}
 	}
 	return ds.Sort()
@@ -748,7 +748,7 @@
 	vdlutil.Vlog.Printf("Parsing package %s %q, dir %s", pkg.Name, pkg.Path, pkg.Dir)
 	files, err := pkg.OpenFiles()
 	if err != nil {
-		errs.Errorf("Can't open vdl files %v, %v", pkg.BaseFileNames, err)
+		errs.Errorf("can't open vdl files %v (%v)", pkg.BaseFileNames, err)
 		return nil
 	}
 	for filename, src := range files {
@@ -818,11 +818,11 @@
 	}
 	target, err := vdl.ReflectTarget(rv)
 	if err != nil {
-		env.Errors.Errorf("Can't create reflect target for %T (%v)", value, err)
+		env.Errors.Errorf("can't create reflect target for %T (%v)", value, err)
 		return
 	}
 	if err := vdl.FromValue(target, vconfig); err != nil {
-		env.Errors.Errorf("Can't convert to %T from %v (%v)", value, vconfig, err)
+		env.Errors.Errorf("can't convert to %T from %v (%v)", value, vconfig, err)
 		return
 	}
 }
diff --git a/lib/vdl/build/build_test.go b/lib/vdl/build/build_test.go
index 3974345..2c9052e 100644
--- a/lib/vdl/build/build_test.go
+++ b/lib/vdl/build/build_test.go
@@ -419,28 +419,28 @@
 		// Non-existent as both import and dir path.
 		{
 			[]string{"noexist"},
-			`Can't resolve "noexist" to any packages`,
+			`can't resolve "noexist" to any packages`,
 		},
 		{
 			[]string{"./noexist"},
-			`Can't resolve "./noexist" to any packages`,
+			`can't resolve "./noexist" to any packages`,
 		},
 		// Invalid package path, as both import and dir path.
 		{
 			[]string{".foo"},
-			`Import path ".foo" is invalid`,
+			`import path ".foo" is invalid`,
 		},
 		{
 			[]string{"foo/.bar"},
-			`Import path "foo/.bar" is invalid`,
+			`import path "foo/.bar" is invalid`,
 		},
 		{
 			[]string{"_foo"},
-			`Import path "_foo" is invalid`,
+			`import path "_foo" is invalid`,
 		},
 		{
 			[]string{"foo/_bar"},
-			`Import path "foo/_bar" is invalid`,
+			`import path "foo/_bar" is invalid`,
 		},
 		{
 			[]string{"../../../../../../.foo"},
@@ -466,7 +466,7 @@
 		},
 		{
 			[]string{"v.io/v23/vdlroot/..."},
-			`Can't resolve "v.io/v23/vdlroot/..." to any packages`,
+			`can't resolve "v.io/v23/vdlroot/..." to any packages`,
 		},
 	}
 	for _, test := range tests {
diff --git a/lib/vdl/compile/type_test.go b/lib/vdl/compile/type_test.go
index 4b64a70..0009a5d 100644
--- a/lib/vdl/compile/type_test.go
+++ b/lib/vdl/compile/type_test.go
@@ -193,4 +193,5 @@
 		{"a", `type Res bool`, vdl.BoolType, ""},
 		{"b", `import "p.kg/a";type Res a".Res`, nil, "syntax error"}}},
 	{"ZeroLengthArray", tp{{"a", `type Res [0]int32`, nil, "negative or zero array length"}}},
+	{"InvalidOptionalFollowedByValidType", tp{{"a", `type Res struct{X ?int32};type x string`, nil, "invalid optional type"}}},
 }
diff --git a/profiles/internal/rpc/stream/manager/listener.go b/profiles/internal/rpc/stream/manager/listener.go
index 191ca22..298d20f 100644
--- a/profiles/internal/rpc/stream/manager/listener.go
+++ b/profiles/internal/rpc/stream/manager/listener.go
@@ -146,17 +146,19 @@
 			return
 		}
 		vlog.VI(1).Infof("New net.Conn accepted from %s (local address: %s)", conn.RemoteAddr(), conn.LocalAddr())
-		vf, err := vif.InternalNewAcceptedVIF(conn, ln.manager.rid, principal, blessings, nil, ln.deleteVIF, opts...)
-		if err != nil {
-			vlog.Infof("Shutting down conn from %s (local address: %s) as a VIF could not be created: %v", conn.RemoteAddr(), conn.LocalAddr(), err)
-			conn.Close()
-			continue
-		}
-		ln.vifs.Insert(vf)
-		ln.manager.vifs.Insert(vf)
+		go func() {
+			vf, err := vif.InternalNewAcceptedVIF(conn, ln.manager.rid, principal, blessings, nil, ln.deleteVIF, opts...)
+			if err != nil {
+				vlog.Infof("Shutting down conn from %s (local address: %s) as a VIF could not be created: %v", conn.RemoteAddr(), conn.LocalAddr(), err)
+				conn.Close()
+				return
+			}
+			ln.vifs.Insert(vf)
+			ln.manager.vifs.Insert(vf)
 
-		ln.vifLoops.Add(1)
-		go vifLoop(vf, ln.q, &ln.vifLoops)
+			ln.vifLoops.Add(1)
+			vifLoop(vf, ln.q, &ln.vifLoops)
+		}()
 	}
 }
diff --git a/lib/vdl/compile/type.go b/lib/vdl/compile/type.go
index 2f577a6..df873c9 100644
--- a/lib/vdl/compile/type.go
+++ b/lib/vdl/compile/type.go
@@ -58,7 +58,10 @@
 	if td.Define(); !env.Errors.IsEmpty() {
 		return
 	}
-	td.Build()
+	if td.Build(); !env.Errors.IsEmpty() {
+		return
+	}
+	td.AttachDoc()
 	// TODO(toddw): should we disallow inter-file cyclic type dependencies?  That
 	// might be an issue for generated C++.
 }
@@ -316,27 +319,30 @@
 				base, err := b.base.Built()
 				if err != nil {
 					td.env.prefixErrorf(file, b.ptype.Pos(), err, "%s base type invalid", def.Name)
-					continue // keep going to catch more errors
+					return
 				}
 				def.BaseType = base
 			}
 			t, err := b.pending.Built()
 			if err != nil {
 				td.env.prefixErrorf(file, def.Pos, err, "%s invalid", def.Name)
-				continue // keep going to catch more errors
+				return
 			}
 			def.Type = t
 			addTypeDef(def, td.env)
 		}
 	}
-	// Make another pass to fill in doc and doc suffix slices for enums, structs
-	// and unions.  Typically these are initialized in makeTypeDefBuilder, based
-	// on the underlying parse data.  But type definitions based on other named
-	// types can't be updated until the base type is actually compiled.
-	//
-	// TODO(toddw): This doesn't actually attach comments from the base type, it
-	// just leaves everything empty.  This is fine for now, but we should revamp
-	// the vdl parsing / comment attaching strategy in the future.
+}
+
+// AttachDoc makes another pass to fill in doc and doc suffix slices for enums,
+// structs and unions.  Typically these are initialized in makeTypeDefBuilder,
+// based on the underlying parse data.  But type definitions based on other
+// named types can't be updated until the base type is actually compiled.
+//
+// TODO(toddw): This doesn't actually attach comments from the base type, it
+// just leaves everything empty.  This is fine for now, but we should revamp the
+// vdl parsing / comment attaching strategy in the future.
+func (td typeDefiner) AttachDoc() {
 	for _, file := range td.pkg.Files {
 		for _, def := range file.TypeDefs {
 			switch t := def.Type; t.Kind() {